﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Configuration;
using System.Net.Http;
using System.Media;
using Newtonsoft.Json;
using Seagull.BarTender.Print;

namespace Teemo
{
    public class GetData
    {
        public Data data { get; set; }
    }

    public class Data
    {
        public string WONumber { get; set; }
        public Product Product { get; set; }
    }

    public class Product
    {
        public string Family { get; set; }
        public string ModelName { get; set; }
        public string CodeAASPV { get; set; }
        public string CodeUPC { get; set; }
        public string CodeAffiliate { get; set; }
        public string CartonGTIN { get; set; }
        public int GiftInCartonQty { get; set; }
        public CartonLabelFile CartonLabelFile { get; set; }
        public double GiftWeightMin { get; set; }
        public double GiftWeightMax { get; set; }
        public bool IsGiftWeight { get; set; }
    }

    public class CartonLabelFile
    {
        public string File { get; set; }
    }

    public class GetCarton
    {
        public DataCarton data { get; set; }
    }

    public class DataCarton
    {
        public string cartonSn { get; set; }
        public string[] giftSn { get; set; }
    }

    public partial class Teemo : Form
    {
        private Thread MainThread;
        private string Host = ConfigurationManager.AppSettings["Host"];
        private string UserID = ConfigurationManager.AppSettings["UserID"];
        private string StationID = ConfigurationManager.AppSettings["StationID"];
        private string Printer = ConfigurationManager.AppSettings["Printer"];
        private int SNLength = int.Parse(ConfigurationManager.AppSettings["SNLength"]);
        private int Delay = int.Parse(ConfigurationManager.AppSettings["Delay"]);
        private string[] Args;
        private HttpClient Client = new HttpClient();
        private Random Rd = new Random();
        private GetData DataStored;
        private GetCarton CartonStored;
        private int GiftQty;
        private string DataInput = "";
        private string LastInput = "";
        private string LabelFile;
        private List<string> GiftInput = new List<string>();
        private List<string> GiftPrint = new List<string>();
        private List<string> GiftHistory = new List<string>();
        private List<string> SubLabel = new List<string>();
        private Engine engine;
        private LabelFormatDocument label;
        private Result result;

        public Teemo()
        {
            InitializeComponent();
        }

        private async Task<string[]> Post(string url, string payload)
        {
            HttpContent content = new StringContent(payload, System.Text.Encoding.UTF8, "application/json");
            var message = await Client.PostAsync(url, content);
            var result = await message.Content.ReadAsStringAsync();
            return new string[] { message.StatusCode.ToString(), result };
        }

        private async Task<string> GetLabel(string url, string name)
        {
            var message = await Client.GetAsync(url);
            using (var result = new FileStream(name, FileMode.Create))
            {
                await message.Content.CopyToAsync(result);
            }
            return message.StatusCode.ToString();
        }

        private void DownloadLabel()
        {
            try
            {
                var t = Task.Run(() => GetLabel(Host + DataStored.data.Product.CartonLabelFile.File, LabelFile));
                t.Wait();
                if (!t.Result.Equals("OK"))
                {
                    DownloadLabel();
                }
            }
            catch
            {
                DownloadLabel();
            }
        }

        private void GetProductInfo()
        {
            try
            {
                string payload = "{\"data_type\":\"GET_PRODUCT_INFO\",\"dut_id\":\"none\",\"station_id\":\"" + StationID + "\",\"data\":null}";
                var t = Task.Run(() => Post(Host + "/shopfloor/api/sendrecvdatasfc/", payload));
                t.Wait();
                if (t.Result[0].Equals("OK"))
                {
                    DataStored = JsonConvert.DeserializeObject<GetData>(t.Result[1]);
                }
                else
                {
                    GetProductInfo();
                }
            }
            catch
            {
                GetProductInfo();
            }
        }

        private void GetCartonInfo()
        {
            try
            {
                string payload = "{\"wo\":\"" + DataStored.data.WONumber + "\"}";
                var t = Task.Run(() => Post(Host + "/shopfloor/carton/getCartonOpenByWO/", payload));
                t.Wait();
                if (t.Result[0].Equals("OK"))
                {
                    CartonStored = JsonConvert.DeserializeObject<GetCarton>(t.Result[1]);
                    GiftQty = CartonStored.data.giftSn.Length;
                    for (int i = 0; i < GiftQty; i++)
                    {
                        GiftPrint.Add(CartonStored.data.giftSn[i]);
                    }
                }
                else
                {
                    GetCartonInfo();
                }
            }
            catch
            {
                GetCartonInfo();
            }
        }

        private void InputGift()
        {
            try
            {
                int time = Rd.Next(999);
                double weight = DataStored.data.Product.IsGiftWeight ? Rd.Next((int)DataStored.data.Product.GiftWeightMin, (int)DataStored.data.Product.GiftWeightMax) + Math.Round(Rd.NextDouble(), 1) : 0.0;
                string payload = "{\"station_name\":\"" + Args[1] + "\",\"line_id\":\"" + Args[0] + "\",\"station_index\":" + int.Parse(Args[2]) + ",\"wo\":\"" + DataStored.data.WONumber + "\",\"giftSn\":\"" + GiftInput[0] + "\",\"cartonSn\":\"" + CartonStored.data.cartonSn + "\",\"User\":\"" + UserID + "\",\"Description\":\"Scale\",\"Result\":true,\"TotalTime\":" + time + ",\"data\":[{\"id\":\"GiftWeight\",\"name\":\"Scale The Gift\",\"step_name\":\"Gift_Weight\",\"result\":\"PASS\",\"value\":\"" + weight + "\",\"lower_limit\":\"" + DataStored.data.Product.GiftWeightMin + "\",\"upper_limit\":\"" + DataStored.data.Product.GiftWeightMax + "\",\"unit\":\"g\",\"time\":" + time + ",\"enable\":true}]}";
                var t = Task.Run(() => Post(Host + "/shopfloor/carton/inputGiftToCarton/", payload));
                t.Wait();
                if (t.Result[0].Equals("OK"))
                {
                    SystemSounds.Asterisk.Play();
                    GiftPrint.Add(GiftInput[0]);
                    GiftInput.RemoveAt(0);
                    GiftQty++;
                    if (GiftQty >= DataStored.data.Product.GiftInCartonQty)
                    {
                        PrintCarton(CartonStored.data.cartonSn);
                        GiftPrint.Clear();
                        GetCartonInfo();
                    }

                }
                else
                {
                    InputGift();
                }
            }
            catch
            {
                InputGift();
            }
        }

        private void PrintCarton(string cartonSn)
        {
            try
            {
                if (SubLabel.Contains("lblModelName") && DataStored.data.Product.ModelName != null)
                {
                    label.SubStrings["lblModelName"].Value = DataStored.data.Product.ModelName;
                }
                if (SubLabel.Contains("lblCodeAffiliate") && DataStored.data.Product.CodeAffiliate != null)
                {
                    label.SubStrings["lblCodeAffiliate"].Value = DataStored.data.Product.CodeAffiliate;
                }
                if (SubLabel.Contains("lblCodeUPC") && DataStored.data.Product.CodeUPC != null)
                {
                    label.SubStrings["lblCodeUPC"].Value = DataStored.data.Product.CodeUPC;
                }
                if (SubLabel.Contains("lblCartonQty"))
                {
                    label.SubStrings["lblCartonQty"].Value = GiftQty.ToString();
                }
                for (int i = 0; i < GiftQty; i++)
                {
                    string name = "lblGift" + (i + 1);
                    if (SubLabel.Contains(name))
                    {
                        label.SubStrings[name].Value = GiftPrint[i];
                    }
                }
                if (SubLabel.Contains("lblCartonID"))
                {
                    label.SubStrings["lblCartonID"].Value = cartonSn;
                }
                if (SubLabel.Contains("lblCartonGTIN") && DataStored.data.Product.CartonGTIN != null)
                {
                    label.SubStrings["lblCartonGTIN"].Value = DataStored.data.Product.CartonGTIN;
                }
            }
            catch { }
            finally
            {
                result = label.Print();
                if ((int)result == 2)
                {
                    MessageBox.Show("Lỗi Trong Quá Trình In Tem\r\nHãy Khởi Động Lại Máy Tính & Máy In!");
                }
            }
        }

        private void Loop()
        {
            while (true)
            {
                Thread.Sleep(Delay);
                if (GiftInput.Count > 0)
                {
                    InputGift();
                }
            }
        }

        private void First_Setup(object obj, EventArgs arg)
        {
            Args = StationID.Split('_');
            GetProductInfo();
            LabelFile = DataStored.data.Product.CartonLabelFile.File.Substring(DataStored.data.Product.CartonLabelFile.File.LastIndexOf('/') + 1);
            if (DataStored.data.Product.Family.Equals("400"))
            {
                DownloadLabel();
                GetCartonInfo();
                try
                {
                    engine = new Engine(true);
                    label = engine.Documents.Open(Directory.GetCurrentDirectory() + "\\" + LabelFile);
                    foreach (SubString sub in label.SubStrings)
                    {
                        SubLabel.Add(sub.Name);
                    }
                    engine.Window.Visible = false;
                    label.PrintSetup.PrinterName = Printer;
                    label.PrintSetup.IdenticalCopiesOfLabel = 1;
                }
                catch
                {
                    MessageBox.Show("Hãy Kiểm Tra Lại File Tem Hoặc Tên Máy In!");
                }
                MainThread = new Thread(Loop);
                MainThread.IsBackground = true;
                MainThread.Start();
            }
            else
            {
                MessageBox.Show("Model Này Chưa Được Hỗ Trợ!");
            }
        }

        private void Key_Pressed(object obj, KeyPressEventArgs arg)
        {
            if (arg.KeyChar >= 48 && arg.KeyChar <= 57 || arg.KeyChar >= 65 && arg.KeyChar <= 90 || arg.KeyChar >= 97 && arg.KeyChar <= 122)
            {
                DataInput += arg.KeyChar;
            }
            if (arg.KeyChar == (char)Keys.Enter)
            {
                DataInput = DataInput.ToUpper();
                if (DataInput.StartsWith(DataStored.data.Product.CodeAASPV) && DataInput.Length == SNLength && !LastInput.Equals(DataInput) && !GiftHistory.Contains(DataInput))
                {
                    LastInput = DataInput;
                    GiftInput.Add(LastInput);
                    if (GiftHistory.Count >= 24)
                    {
                        GiftHistory.Clear();
                    }
                    GiftHistory.Add(LastInput);
                }
                DataInput = "";
            }
        }

        private void Finish(object obj, FormClosingEventArgs arg)
        {
            try
            {
                label.Close(SaveOptions.DoNotSaveChanges);
                engine.Stop();
            }
            catch { }
        }
    }
}
